@@ -18,8 +18,10 @@
 
 #include "chewing-utf8-util.h"
 #include "global.h"
-#include "hash.h"
-#include "dict.h"
+#include "hash-private.h"
+#include "dict-private.h"
+#include "tree-private.h"
+#include "userphrase-private.h"
 #include "private.h"
 
 extern int chewing_lifetime;
@@ -54,10 +56,9 @@ static int LoadOriginalFreq( const uint16 phoneSeq[], const char wordSeq[], int
 		GetPhraseFirst( phrase, pho_id );
 		do {
 			/* find the same phrase */
-			if ( ! memcmp( 
+			if ( ! strcmp(
 				phrase->phrase, 
-				wordSeq, 
-				len * MAX_UTF8_SIZE * sizeof( char ) ) ) { 
+				wordSeq ) ) {
 				retval = phrase->freq;	
 				free( phrase );
 				return retval;
